@@ -42,10 +42,9 @@ from clusterondemandconfig.configuration_validation import (
 )
 
 from .cluster import Cluster
-from .configuration import awscommon_ns, awsinstancetype_ns
-from .ec2connection import establish_connection_to_ec2
+from .configuration import awscommon_ns
+from .ec2connection import create_ec2_resource_client, establish_connection_to_aws
 from .images import AWSImageSource, findimages_ns
-from .instancetype import NoTypesXMLError
 from .vpc import create_in_existing_vpc, create_vpc
 
 log = logging.getLogger("cluster-on-demand")
@@ -56,8 +55,7 @@ HR = "---------------------------------------------------------------------"
 config_ns = ConfigNamespace("aws.cluster.create", "cluster creation parameters")
 config_ns.import_namespace(awscommon_ns)
 config_ns.import_namespace(findimages_ns)
-config_ns.override_imported_parameter("version", default="9.1")
-config_ns.import_namespace(awsinstancetype_ns)
+config_ns.override_imported_parameter("version", default="9.2")
 config_ns.import_namespace(clusterondemand.configuration.clustercreate_ns)
 config_ns.import_namespace(clusterondemand.configuration.cmd_debug_ns)
 config_ns.override_imported_parameter(
@@ -463,14 +461,7 @@ class ClusterCreate(ClusterCommandBase):
         if config["validate_parameters"]:
             self._validate_cluster_names()
             self._validate_aws_access_credentials()
-            try:
-                self._validate_instance_types()
-            except NoTypesXMLError as e:
-                raise UserReportableException(
-                    f"Failed to read types file from '{e.url}'. You can: use --types-url to set a different URL, "
-                    f"use --fetch-from-aws to use the (slower) AWS pricing API directly or use "
-                    f"--no-validate-parameters to disable all parameter validations (use with care)."
-                )
+            self._validate_instance_types()
         else:
             log.warning("Ignoring parameter validation. Use --validate-parameters to enable it.")
 
@@ -565,7 +556,8 @@ class ClusterCreate(ClusterCommandBase):
     def run(self):
         self._validate_params()
 
-        ec2, ec2c = establish_connection_to_ec2()
+        aws_session = establish_connection_to_aws()
+        ec2, ec2c = create_ec2_resource_client(aws_session)
 
         if config["existing_vpc_id"]:
             self._validate_params_existing_vpc(ec2, ec2c)
@@ -607,16 +599,22 @@ class ClusterCreate(ClusterCommandBase):
             if prefix:
                 names = ["%s-%s" % (prefix, name) for name in names]
 
-        conflicting_names = {cluster.name for cluster in Cluster.find(ec2, ec2c, names)}
+        conflicting_names = {cluster.name for cluster in Cluster.find(aws_session, names)}
 
         if conflicting_names:
             raise UserReportableException("Cluster with this names already exist: %s" %
                                           " ".join(conflicting_names))
 
         head_node_image = AWSImageSource.pick_head_node_image_using_options(config)
+        if not head_node_image.version and config["run_cm_bright_setup"]:
+            log.warning(
+                f"Using custom image: {head_node_image.uuid} with parameter run_cm_bright_setup set to 'yes'."
+                f" Probably it was set by mistake because a custom image might not have necessary files"
+                f" to run cm-bright-setup. Consider using --run-cm-bright-setup=no to "
+            )
 
         requested = [
-            Cluster(ec2, ec2c, name, head_node_image)
+            Cluster(aws_session, name, head_node_image)
             for name in names
         ]
 
@@ -655,18 +653,18 @@ class ClusterCreate(ClusterCommandBase):
             number_of_stored_ips = 0
             with open(config["store_head_node_ip"], "w") as f:
                 for cluster in requested:
-                    if cluster.head_node and cluster.head_node.public_ip_address:
+                    if cluster.primary_head_node and cluster.primary_head_node.public_ip_address:
                         number_of_stored_ips += 1
-                        f.write(cluster.head_node.public_ip_address + " " + cluster.name + "\n")
+                        f.write(cluster.primary_head_node.public_ip_address + " " + cluster.name + "\n")
             log.info("Stored %s IP(s) in %s" %
                      (number_of_stored_ips, config["store_head_node_ip"]))
         if config["store_head_node_id"]:
             number_of_stored_ids = 0
             with open(config["store_head_node_id"], "w") as f:
                 for cluster in requested:
-                    if cluster.head_node and cluster.head_node.id:
+                    if cluster.primary_head_node and cluster.primary_head_node.id:
                         number_of_stored_ids += 1
-                        f.write(cluster.head_node.id + " " + cluster.name + "\n")
+                        f.write(cluster.primary_head_node.id + " " + cluster.name + "\n")
             log.info("Stored %s ID(s) in %s" %
                      (number_of_stored_ids, config["store_head_node_id"]))
         if cluster_creator.failed:
@@ -739,7 +737,7 @@ class ClusterCreator(object):
 
             for cluster in clusters_to_create:
                 try:
-                    # cluster.vpc and cluster.head_node will be written in this call
+                    # cluster.vpc and cluster.primary_head_node will be written in this call
                     if config["existing_vpc_id"]:
                         create_in_existing_vpc(cluster)
                     else:
@@ -787,26 +785,26 @@ class ClusterCreator(object):
         log.info(HR)
         log.info("Time it took:   %s" % delta_string)
 
-        if len(to_create_names) == 1 and len(failed_names) == 0 and self.requested[0].head_node:
+        if len(to_create_names) == 1 and len(failed_names) == 0 and self.requested[0].primary_head_node:
             """ One cluster OK """
             head_node_ip = None
             if config["head_node_assign_public_ip"]:
-                head_node_ip = self.requested[0].head_node.public_ip_address
+                head_node_ip = self.requested[0].primary_head_node.public_ip_address
             else:
-                head_node_ip = self.requested[0].head_node.private_ip_address
+                head_node_ip = self.requested[0].primary_head_node.private_ip_address
             log.info(f"  SSH string:   'ssh root@{head_node_ip}'")
         else:
             log.info("   to create:   %s" % len(to_create_names))
             log.info("     created:   %s" % len([cluster for cluster in self.requested
-                                                 if cluster.head_node]))
+                                                 if cluster.primary_head_node]))
             if len(self.failed):
                 log.info("      failed:   %s" % len(self.failed))
 
         for r in self.requested:
-            if r.head_node:
+            if r.primary_head_node:
                 # Print in hosts format (easy copy/paste)
-                print("%s  %s" % (r.head_node.public_ip_address, r.name))
-                log.info("Head node ID:   %s" % r.head_node.id)
+                print("%s  %s" % (r.primary_head_node.public_ip_address, r.name))
+                log.info("Head node ID:   %s" % r.primary_head_node.id)
             else:
                 pass
         if config["log_cluster_password"]:
@@ -843,14 +841,14 @@ def start_headnodes(clusters):
     # it looks like newly create VMs get started by default anyway. It's just a matter of waiting for them to be running
     # Not sure why we have this "start" code here. We should consider removing it.
     for cluster in clusters:
-        cluster.head_node.start()
+        cluster.primary_head_node.start()
 
     failed = []
 
     utils.cod_log(log, f"Waiting for the head node{'s' if len(clusters) > 1 else ''} to start", 30)
 
     for cluster in clusters[:]:
-        cluster.head_node.wait_until_running()
+        cluster.primary_head_node.wait_until_running()
 
         try:
             if config["head_node_assign_public_ip"]:
@@ -862,7 +860,7 @@ def start_headnodes(clusters):
                     log.info("Note that you're creating a cluster in an existing VPC, while at the same time "
                              "not ensuring that IGW exists. There's a risk assigning a public IP might fail.")
 
-                cluster.assign_eip()
+                cluster.assign_eips()
             else:
                 utils.cod_log(log,
                               "Will not assign a public IP to the head node "
@@ -907,9 +905,9 @@ def wait_for_cloud_init(clusters):
             # IP
             ip_to_check = None
             if config["head_node_assign_public_ip"]:
-                ip_to_check = cluster.head_node.public_ip_address
+                ip_to_check = cluster.primary_head_node.public_ip_address
             else:
-                ip_to_check = cluster.head_node.private_ip_address
+                ip_to_check = cluster.primary_head_node.private_ip_address
 
             # cmdaemon port
             port = 8081 if config["run_cm_bright_setup"] else 22
